package tfg.sma.core.v2.agent;

import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;

import jade.content.lang.sl.SLCodec;
import jade.core.Agent;
import jade.core.behaviours.DataStore;
import jade.core.behaviours.OneShotBehaviour;
import jade.core.behaviours.ParallelBehaviour;
import jade.domain.*;
import jade.domain.FIPAAgentManagement.*;
import tfg.sma.core.model.AgentDescriptor;
import tfg.sma.core.model.InferenceDescriptor;
import tfg.sma.core.model.Variable;
import tfg.sma.core.v2.behaviours.Asking;
import tfg.sma.core.v2.behaviours.WisdomClaim;
import tfg.sma.core.v2.complex.behaviours.PrimeBehaviour;
import tfg.sma.core.v2.complex.behaviours.SetupBehaviour;
import tfg.sma.core.v2.ontology.TfgSmaOntology;
import tfg.sma.gui.BaseAgentGui;
import tfg.sma.logging.LogFactory;
import tfg.sma.logging.LogMessages;
import tfg.sma.utility.AgentHelpers;


@SuppressWarnings("serial")
public class BaseAgent extends Agent {
	
	public static final String AGENTTYPE = "BaseAgent";
	public static final String AGENTGENERICNAME = "Agent";
	
	private Map<String, Object> variables;	
	private BaseAgentGui agentJadeGui;
	private Boolean registeredDf; //Used mainly to know if the agent has reached the registration easily.
	private AgentDescriptor agentDescriptor; 
	private InferenceDescriptor outputInferenceDescriptor;
	private Logger logger;
	private DataStore dataStore;
	private Long sessionId;
	

	public BaseAgent() {
		super();
		variables = new HashMap<String, Object>();
	}

	@Override
	protected void setup() {
		registeredDf = false;
		super.setup();
		agentFeatures();
	}

	private void agentFeatures() {
			//Setup agent : To perform the steps of the agent initialization.
			SetupBehaviour setupB = new SetupBehaviour(this);					
			//Paralel behaviour -Prime behaviour and asking values ability.						
			ParallelBehaviour parallelB = new ParallelBehaviour(this, ParallelBehaviour.WHEN_ANY);									
			
			//Ontology			
			getContentManager().registerOntology(TfgSmaOntology.getInstance());
			getContentManager().registerLanguage(new SLCodec());
						
			//First setup
			for (Object o : getArguments()) {				
				//Current and future features a base agent may have.
				switch (o.getClass().getSimpleName()) {
				// If the current argument is the inferenceDescriptor.
				case InferenceDescriptor.NAME:
					outputInferenceDescriptor = ((InferenceDescriptor) o);					
					setupB.registerFirstState(new WisdomClaim(outputInferenceDescriptor.getFileName()), SetupBehaviour.WISDOM);					
					break;					
				//Evaluating agentDescriptor stuff	
				case AgentDescriptor.NAME:
					agentDescriptor = ((AgentDescriptor) o);
					//askPeriod - Adding the automatic asking ability to the agent.
					askPeriod(agentDescriptor, parallelB);										
					//Setting what inference may feed this the input values of this agent.: 
					variables.put(Variable.INPUTFIS, agentDescriptor.getInputInferenceId());
										
					//Future features based on the descriptor...					
					break;	
				case LogFactory.NAME:
					//Using the same logger as the hive agent...all agents will log to the same place.
					logger = (Logger) o;
					break;				
				case "Long":
					sessionId = (Long) o;
					break;
				}
			}
			//Last registration step
			setupB.registerState(new OneShotBehaviour() {				
				@Override
				public void action() {
					registrarDF(agentDescriptor);
				}
			}, SetupBehaviour.REGISTRATION);
			
			//Finally we are adding the main behaviour to the parallel one.
			parallelB.addSubBehaviour(new PrimeBehaviour(this));

			//Registering to te Setup behaviour the parallel one.
			setupB.registerState(parallelB, SetupBehaviour.PARALLEL_PRIME_ASK);
			addBehaviour(setupB);		
	
	}
	/**
	 * Grants aumoatic asking to other agents.
	 * @param ad
	 * @param b
	 */
	private void askPeriod(AgentDescriptor ad, ParallelBehaviour b) {		
		long askPeriod =  (long) ad.getAskPeriod()*1000;
		if(!(askPeriod == 0))
			b.addSubBehaviour(new Asking(this, askPeriod));
	}
	 
	public void registeredTakeDown() {
		AgentHelpers.logAgent(this, Level.WARNING, this.getName()+LogMessages.AGENTDOWN.getDescription());
		super.takeDown();
		deRegistrarDF();
	}	
	public void unregisteredTakeDown() {
		AgentHelpers.logAgent(this, Level.WARNING, this.getName()+LogMessages.AGENTDOWN.getDescription());
		super.takeDown();
	}
	public void registrarDF(AgentDescriptor ad) {
		DFAgentDescription dfd = new DFAgentDescription();
		dfd.addServices(AgentHelpers.CannonicalServiceDescription(this, true, false));
		dfd.setName(getAID());
		try {
			DFService.register(this, dfd);
			AgentHelpers.logAgent(this, Level.INFO, getAID().getLocalName()+LogMessages.AGENTREGISTERED.getDescription());
			registeredDf = true;
		} catch (FIPAException e) {

		}
	}
	public void deRegistrarDF() {
		try {
			DFService.deregister(this);
			AgentHelpers.logAgent(this, Level.INFO, getAID().getLocalName()+LogMessages.AGENTUNREGISTERED.getDescription());
		} catch (Exception e) {

		}
	}
	public Map<String, Object> getVariables() {
		return variables;
	}
	public void setVariables(Map<String, Object> variables) {
		this.variables = variables;
	}
	public BaseAgentGui getMyGui() {
		return agentJadeGui;
	}
	public void setMyGui(BaseAgentGui myGui) {
		this.agentJadeGui = myGui;
	}
	@SuppressWarnings("unchecked")
	public Vector<Variable> getAgentOutputVariables() {
		return (Vector<Variable>) this.variables.get(Variable.OUTPUT);
	}
	@SuppressWarnings("unchecked")
	public Vector<Variable> getAgentInputVariables() {
		return (Vector<Variable>) this.variables.get(Variable.INPUT);
	}
	public Boolean getRegisteredDf() {
		return registeredDf;
	}
	public BaseAgentGui getAgentJadeGui() {
		return agentJadeGui;
	}
	public InferenceDescriptor getOutputInferenceDescriptor() {
		return outputInferenceDescriptor;
	}
	public AgentDescriptor getAgentDescriptor() {
		return agentDescriptor;
	}
	public DataStore getDataStore() {
		return dataStore;
	}
	public void setDataStore(DataStore dataStore) {
		this.dataStore = dataStore;
	}
	public Logger getLogger() {
		return logger;
	}
	public Long getSessionId() {
		return sessionId;
	}
	
}
