package tfg.sma.core.v2.agent;

import java.io.IOException;
import java.sql.SQLException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.bind.JAXBException;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.SAXException;
import jade.core.Agent;
import jade.core.Profile;
import jade.core.ProfileImpl;
import jade.core.Runtime;
import jade.core.behaviours.Behaviour;
import jade.core.behaviours.ParallelBehaviour;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.wrapper.AgentContainer;
import tfg.sma.core.model.SmaDescriptor;
import tfg.sma.core.v2.behaviours.BringAgents;
import tfg.sma.ddbb.DbConstants;
import tfg.sma.ddbb.DbManager;
import tfg.sma.gui.HiveAgentGui;
import tfg.sma.logging.LogFactory;
import tfg.sma.logging.LogMessages;
import tfg.sma.utility.AgentHelpers;
import tfg.sma.utility.PlatformHelpers;
import tfg.sma.utility.SmaDescriptorProvider;


@SuppressWarnings("serial")
public class HiveAgent extends Agent {
	
	public static final String AGENTTYPE = "HiveAgent";
	public static final String AGENTGENERICNAME = "Hive";
	
	public static Logger logger;
	private static SmaDescriptor smad;
	private Long sessionId;
	
	public HiveAgent() {
		super();		
		logger = LogFactory.createLog(Class.class.getName(), LogFactory.LOGFOLDERNAME, LogFactory.LOGFILENAME);		
	}
			
	@Override
	protected void setup() {
		AgentHelpers.logAgent(this, Level.INFO, LogMessages.SMASTARTING.getDescription());
		Profile profile = new ProfileImpl(false);		
		smad = null;
		AgentContainer container;
		sessionId = null;
		try {
			//Descriptor well formed and validated.
			smad = SmaDescriptorProvider.getDescriptor();
			AgentHelpers.logAgent(this, Level.INFO, LogMessages.DESCRIPTORFINE.getDescription());															
			//DBManager preparation
			sessionId = initDbManager(smad);
			
		} catch (ParserConfigurationException | SAXException | IOException | JAXBException e) {
			AgentHelpers.logAgent(this, Level.SEVERE, LogMessages.DESCRIPTORTROUBLE.getDescription()+e);
					
		}catch (SQLException | ClassNotFoundException e) {
			//DDBB trouble
			AgentHelpers.logAgent(this, Level.SEVERE, LogMessages.DDBBTROUBLE.getDescription()+e);
		}
		finally {
			if(smad == null) {
				//Nothing shall continue from this point..It's mandatory the descriptor, at least, being well formed and validated. JADE runtime is terminated.
				AgentHelpers.logAgent(this, Level.SEVERE, LogMessages.TERMINATINGJADE.getDescription());
				PlatformHelpers.shutDownJadePlatform(this);
			}else {
				//Container creation and agent bringing
				profile.setParameter(Profile.CONTAINER_NAME, "Hive");
				container = Runtime.instance().createAgentContainer(profile);
				registrarDF();
				
				//Bringing the agents to life.
				//addBehaviour(new BringAgents(container, smad, sessionId!=null?sessionId:new Long(-1)));
				
				//POC
				ParallelBehaviour palb = new ParallelBehaviour();
				palb.addSubBehaviour(new BringAgents(container, smad, sessionId!=null?sessionId:new Long(-1)));
				
				
				palb.addSubBehaviour(new Behaviour() {
					private int state;
					@Override
					public boolean done() {
						// TODO Auto-generated method stub
						return state == 1;
					}
					
					@Override
					public void action() {
						try {
							state = 0;							
							DFAgentDescription[] agents = null;
							agents = PlatformHelpers.getAllBaseAgents(myAgent);							
							long endtime = System.currentTimeMillis() + new Long(2000);
							while(smad.getListOfAgents().size() != agents.length && endtime >=System.currentTimeMillis() ) {
								agents = PlatformHelpers.getAllBaseAgents(myAgent);								
							}
							
							new HiveAgentGui((HiveAgent)myAgent, agents).showGui();;							
							state = 1;
							 							
						} catch (FIPAException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
						
					}
				});
				
				
				
				
				addBehaviour(palb);
			}
		}		
	}
		
	
	public void registrarDF() {		 
		DFAgentDescription dfd = new DFAgentDescription(); 		
		dfd.addServices(AgentHelpers.CannonicalServiceDescription(this, true, false));
		dfd.setName(getAID());		
		try {
			DFService.register(this,  dfd);
			AgentHelpers.logAgent(this, Level.INFO, LogMessages.HIVEAGENTREGISTERED.getDescription());			
		}catch(FIPAException e) {
			AgentHelpers.logAgent(this, Level.WARNING, LogMessages.TROUBLEUNREGISTERINGAGENT.getDescription()+e);
		}	
	}	
	public void deRegistrarDF() {
		try {
			DFService.deregister(this);
		}catch(FIPAException e) {
			AgentHelpers.logAgent(this, Level.WARNING, LogMessages.TROUBLEUNREGISTERINGAGENT.getDescription()+e);
		}	
	}
	@Override
	public void takeDown() {
		super.takeDown();
		deRegistrarDF();
	}

	private Long initDbManager(SmaDescriptor smad) throws SQLException, ClassNotFoundException {		
		//Connection to the Database. If it fails will be logged but the platform will be running wihout  storing data into the DDBB.
		DbManager.getInstance(smad.getDataBase()).TestDatabase(); //The first time, since the object will be instantiated, it's interesting to pass its parameters obtained from the descriptor.
		DbManager.getInstance().createDDBBTables();
		AgentHelpers.logAgent(this, Level.INFO, LogMessages.DDBBFINE.getDescription());
		return DbManager.getInstance().newTfgSmaSession(smad.getSimulation(), DbConstants.STARTED);
	}
	
	
	public Logger getLogger() {
		return logger;
	}

	public Long getSessionId() {
		return sessionId;
	}
	
	

}
