package tfg.sma.core.v2.behaviours;

import java.util.Vector;
import java.util.logging.Level;
import jade.content.ContentElementList;
import jade.core.behaviours.Behaviour;
import jade.core.behaviours.DataStore;
import jade.lang.acl.ACLMessage;
import tfg.sma.core.model.Variable;
import tfg.sma.core.v2.agent.BaseAgent;
import tfg.sma.core.v2.ontology.AgentInformationPredicate;
import tfg.sma.core.v2.ontology.PropertyConcept;
import tfg.sma.core.v2.ontology.VariablesPredicate;
import tfg.sma.logging.LogMessages;
import tfg.sma.utility.AgentHelpers;

@SuppressWarnings("serial")
public class Answer extends Behaviour {

	
	public static final int OK = 0;
	public static final int KO = -1;
	private int state;
	
	public Answer(DataStore ds) {
		this.setDataStore(ds);
	}
	
	@Override
	public void action() {
		state = KO;		
		BaseAgent ba = (BaseAgent) myAgent;
		ACLMessage inputMessage = ((ACLMessage) this.getDataStore().get(Receiver.INPUTMESSAGE));
		ACLMessage answer = inputMessage.createReply();
		Object results = this.getDataStore().get(Inference.RESULT);
		
		if(results != Variable.NULL){			
			VariablesPredicate vars = new VariablesPredicate();	
			AgentInformationPredicate aip = new AgentInformationPredicate();
			ContentElementList cel = new ContentElementList();
						
			//Enriching the answer.
			addVariablesTotheAnswer(ba, results, vars, cel);
			addContextPropertiesToTheAnswer(ba,aip,cel);
			
			//Filling the answer message by ontology means.			
			try {								
				ba.getContentManager().fillContent(answer, cel);
			} catch (Exception e) {
				AgentHelpers.logAgent(ba, Level.WARNING, LogMessages.FILLMESSAGEERROR.getDescription()+e);				
			}			
			answer.setPerformative(ACLMessage.INFORM_REF);
		}else {			
			answer.setPerformative(ACLMessage.FAILURE);
			answer.setContent(Variable.NULL);
		} 							
		AgentHelpers.logAgent(ba, Level.INFO, ba.getAID().getLocalName()+LogMessages.ANSWERING.getDescription()+"TO: "+inputMessage.getSender().getLocalName());		
		myAgent.send(answer);
		state = OK;		
	}

	@SuppressWarnings("unchecked")
	private void addVariablesTotheAnswer(BaseAgent ba, Object results, VariablesPredicate vars, ContentElementList cel ) {
		//Adding input variables of this agent, to let know the requestor agent the input values used.
		for(Variable v: ba.getAgentInputVariables()) {
			vars.getVariables().add(v);				
		}
		//Adding inference results 
		for(Variable v: (Vector<Variable>)results) {
			vars.getVariables().add(v);			
		}			
		cel.add(vars);	
	}
	
	private void addContextPropertiesToTheAnswer(BaseAgent ba, AgentInformationPredicate aip, ContentElementList cel ) {			
		aip.getAgentProperties().add(new PropertyConcept(PropertyConcept.FCLFILENAME, ba.getOutputInferenceDescriptor().getFileName()));
		aip.getAgentProperties().add(new PropertyConcept(PropertyConcept.AGENTNAME, ba.getAgentDescriptor().getName()));
		aip.getAgentProperties().add(new PropertyConcept(PropertyConcept.INFERENCEID, Integer.toString(ba.getOutputInferenceDescriptor().getId())));
		cel.add(aip);		
	}	
	
	@Override
	public boolean done() {
		return state == OK;
	}
	
	@Override
	public int onEnd() {		
		return state;
	}
}
