package tfg.sma.core.v2.behaviours;

import java.sql.SQLException;
import java.util.Vector;
import java.util.logging.Level;
import jade.content.ContentElement;
import jade.content.ContentElementList;
import jade.content.lang.Codec.CodecException;
import jade.content.onto.OntologyException;
import jade.core.behaviours.Behaviour;
import jade.core.behaviours.DataStore;
import jade.lang.acl.ACLMessage;
import tfg.sma.core.model.Variable;
import tfg.sma.core.v2.agent.BaseAgent;
import tfg.sma.core.v2.ontology.AgentInformationPredicate;
import tfg.sma.core.v2.ontology.PropertyConcept;
import tfg.sma.core.v2.ontology.VariablesPredicate;
import tfg.sma.ddbb.DbManager;
import tfg.sma.logging.LogMessages;
import tfg.sma.utility.AgentHelpers;

@SuppressWarnings("serial")
public class Answered extends Behaviour {

	public static final int OK = 0;
	public static final int KO = -1;
	private int state;
	
	public Answered(DataStore ds) {
		this.setDataStore(ds);
	}
		
	@Override
	public void action() {
		state = KO;
		BaseAgent ba = (BaseAgent) myAgent;
		ACLMessage m = (ACLMessage) this.getDataStore().get(Receiver.INPUTMESSAGE);
		AgentHelpers.logAgent(ba, Level.INFO, ba.getAID().getLocalName()+LogMessages.ANSWERRECEIVED.getDescription()+"|"+m.getContent());

		switch(m.getContent()) {
		case Variable.NULL:
			//Nothing to do
			break;
		default:
			try {				
				//Processing the received data from the agent.
				ContentElementList receivedData= (ContentElementList) ba.getContentManager().extractContent(m);				
				for(int i=0;i<receivedData.size();i++) {
					
					ContentElement ce = receivedData.get(i);
					switch(ce.getClass().getSimpleName()) {
						case VariablesPredicate.NAME:
							populateVariables(ba,ce);
							persistInteraction(ba,receivedData);
						break;
						case AgentInformationPredicate.NAME:
											
						break;
					}					
				}								
			} catch (CodecException | OntologyException e) {
				AgentHelpers.logAgent(ba, Level.WARNING, LogMessages.EXTRACTIONERROR.getDescription()+e);				
			}
			
			if(ba.getAgentJadeGui() != null)
				ba.getAgentJadeGui().notifyGuiNewValues();			
			break;
		}		
		state = OK;
	}

	private void populateVariables(BaseAgent ba, ContentElement content) {
		VariablesPredicate variables =(VariablesPredicate) content;								
		for(int a = 0; a<variables.getVariables().size(); a++) {
			Variable v = (Variable) variables.getVariables().get(a);
			if(v.getType().equals(Variable.OUTPUT)) { //The output variables from the other agent will feed the input ones of this agent.
				@SuppressWarnings("unchecked")
				Vector<Variable> currentValues =(Vector<Variable>) ba.getVariables().get(Variable.INPUT);
				for(Variable var :currentValues) {
					if(var.getName().equals(v.getName()))
						var.setValue(v.getValue());
				}	
			}														
		}
	}
	/**
	 * We are processing the varirables here once again...Trying to decouple the agent population and the DDBB data population.
	 * @param ba
	 * @param content
	 */
	private void persistInteraction(BaseAgent ba, ContentElementList receivedData) {			
		if(DbManager.isCONNECTION_OK()) {
			VariablesPredicate variables = null;
			AgentInformationPredicate answererAgent = null;
			
			for(int i=0;i<receivedData.size();i++) {			
				ContentElement ce = receivedData.get(i);
				switch(ce.getClass().getSimpleName()) {
					case VariablesPredicate.NAME:
						variables =(VariablesPredicate) ce;		
					break;
					case AgentInformationPredicate.NAME:
						answererAgent = (AgentInformationPredicate) ce;			
					break;
				}					
			}	
					
			try {				
				String responderAgent = answererAgent.getPropertyByName(PropertyConcept.AGENTNAME).getValue();
				String responderInferenceId = answererAgent.getPropertyByName(PropertyConcept.INFERENCEID).getValue();
				String responderFclFileName = answererAgent.getPropertyByName(PropertyConcept.FCLFILENAME).getValue();
				
				Long interactionId = DbManager.getInstance().newTfgSmaInteraction(ba.getSessionId(), ba.getAgentDescriptor().getName(), responderAgent!=null?responderAgent:"", Integer.parseInt(responderInferenceId), responderFclFileName);
				if(variables != null) {
					for(int a = 0; a<variables.getVariables().size(); a++) {
						Variable v = (Variable) variables.getVariables().get(a);
						DbManager.getInstance().newTfgSmaInterVariable(interactionId, v.getName(), v.getType(), (Double)v.getValue());
					}	
				}
													
			} catch (ClassNotFoundException | SQLException e) {
				AgentHelpers.logAgent(ba, Level.WARNING, LogMessages.TROUBLEINSERTINGINTERACTION.getDescription()+e);
			} 
		}
				
	}
	
		
	@Override
	public boolean done() {
		return state == OK ;
	}
	
	@Override
	public int onEnd() {		
		return state;
	}
	
}
