package tfg.sma.core.v2.behaviours;

import java.util.Vector;
import java.util.logging.Level;
import jade.core.Agent;
import jade.core.behaviours.WakerBehaviour;
import jade.domain.FIPAException;
import jade.domain.FIPANames;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.lang.acl.ACLMessage;
import tfg.sma.core.v2.agent.BaseAgent;
import tfg.sma.core.v2.ontology.TfgSmaOntology;
import tfg.sma.logging.LogMessages;
import tfg.sma.utility.AgentHelpers;
import tfg.sma.utility.PlatformHelpers;

@SuppressWarnings("serial")
public class Asking extends WakerBehaviour {
	
	public static final int OK = 0;
	public static final int KO = -1;	
	private int status;
	private long nextAskingPeriod;
	
	public Asking(Agent a, long timeout) {
		super(a, timeout);
		nextAskingPeriod = timeout;
	}
		
	@Override
	protected void onWake() {
		status = KO;
		BaseAgent ba = (BaseAgent) myAgent;
		AgentHelpers.logAgent(ba, Level.INFO, ba.getAID().getLocalName()+LogMessages.ASKINGVALUES.getDescription());		
		
		ACLMessage askMessage = new ACLMessage(ACLMessage.REQUEST);
		//Best place to be?
		askMessage.setLanguage(FIPANames.ContentLanguage.FIPA_SL);
		askMessage.setOntology(TfgSmaOntology.ONTOLOGYNAME);
		
		DFAgentDescription[] agents = null;
		try {
			agents = PlatformHelpers.getMatchingAgents(ba, false); //Strict agent search based on the configured inferenceoutputid
			
			if(agents.length == 0)
				agents = PlatformHelpers.getMatchingAgents(ba, true); //Relaxed agent search based only on variable names
			
			AgentHelpers.logAgent(ba, Level.INFO, ba.getAID().getLocalName()+LogMessages.AGENTSFOUND.getDescription()+": "+agents.length);
		} catch (FIPAException e) {
			AgentHelpers.logAgent(ba, Level.WARNING, LogMessages.ASKINGTROUBLE.getDescription()+e);
		}
		
		Vector<DFAgentDescription> v = new Vector<DFAgentDescription>();		
		for(int a=0;a<agents.length;a++) {
			if(!agents[a].getName().equals(myAgent.getAID()))
				v.add(agents[a]);
		}
		
		if(v.size() > 0) {		
			//TODO : Criteria to be revised.
			int n = (int) (Math.random() * v.size());			 
			askMessage.addReceiver(v.get(n).getName());
			AgentHelpers.logAgent(ba, Level.INFO, ba.getAID().getLocalName()+LogMessages.REQUESTINGTO.getDescription()+": "+v.get(n).getName().getLocalName());
			myAgent.send(askMessage);			
			status = OK;	
		}else {
			AgentHelpers.logAgent(ba, Level.INFO, ba.getAID().getLocalName()+LogMessages.NOAGENTSFOUND.getDescription());
		}
		
	}
	
	@Override
	public int onEnd() {
		myAgent.addBehaviour(new Asking(myAgent,nextAskingPeriod));
		return status;

	}
}
