package tfg.sma.core.v2.behaviours;

import java.sql.SQLException;
import java.util.Vector;
import java.util.logging.Level;
import jade.core.behaviours.Behaviour;
import jade.core.behaviours.DataStore;
import net.sourceforge.jFuzzyLogic.FIS;
import tfg.sma.core.model.Variable;
import tfg.sma.core.v2.agent.BaseAgent;
import tfg.sma.ddbb.DbConstants;
import tfg.sma.ddbb.DbManager;
import tfg.sma.logging.LogMessages;
import tfg.sma.utility.AgentHelpers;

@SuppressWarnings("serial")
public class Inference extends Behaviour{


	public static final int OK = 0;
	public static final int KO = -1;
	public static final String RESULT = "Result";
		
	private FIS fs;
	private int state;
	private boolean manual;
	
	public Inference(DataStore dsm, boolean manual) {		
		this.setDataStore(dsm);	
		this.manual = manual;
	}
	
	
	@Override
	public void action() {		
		
		BaseAgent ba = (BaseAgent) myAgent;		
		try {						
			runJFuzzyLogicInference();
			Vector<Variable> result = new Vector<Variable>();
			for(Variable v:ba.getAgentOutputVariables()) {									
				result.add(new Variable(Variable.OUTPUT, v.getName(), fs.getVariable(v.getName()).getValue() ));					
				v.setValue(fs.getVariable(v.getName()).getValue());
			}
						
			this.getDataStore().put(RESULT, result);
			
			if(ba.getAgentJadeGui() != null)
				ba.getAgentJadeGui().notifyGuiNewValues();
						
			if(manual == true)
				persistLocalInteraction(ba);
									
			state = OK;			
		} catch (Exception e) {
			state = KO;
			AgentHelpers.logAgent(ba, Level.WARNING, myAgent.getAID().getLocalName()+LogMessages.INFERENCEERROR.getDescription()+e);			
			this.getDataStore().put(RESULT, Variable.NULL);						
		}		
					
	}
	
	private void runJFuzzyLogicInference() throws Exception {		
		fs = (FIS) ((BaseAgent)myAgent).getVariables().get(Variable.FUZZYISYSTEM);		
		Boolean varOk = true;		
		
		for(Variable v:((BaseAgent)myAgent).getAgentInputVariables()) {
			if(v.getValue().equals(Variable.NULL)) {
				varOk = false;		
				break;
			}else {									
				fs.setVariable(v.getName(), (Double)v.getValue());
			}						
		}		
		if(varOk) {
			fs.evaluate();			
		}else throw new Exception(LogMessages.NODATATOINFERE.getDescription());		
	}
	
	
	private void persistLocalInteraction(BaseAgent ba) {					
		try {
			if(DbManager.isCONNECTION_OK()) {
				Long interactionId = DbManager.getInstance().newTfgSmaInteraction(ba.getSessionId(), DbConstants.SSSOURCE_HUMAN, ba.getAgentDescriptor().getName(), ba.getOutputInferenceDescriptor().getId(), ba.getOutputInferenceDescriptor().getFileName());
				for(Variable v:ba.getAgentInputVariables()) {
					DbManager.getInstance().newTfgSmaInterVariable(interactionId, v.getName(), v.getType(), Double.parseDouble(v.getValue().toString()));
				}
				for(Variable v:ba.getAgentOutputVariables()) {
					DbManager.getInstance().newTfgSmaInterVariable(interactionId, v.getName(), v.getType(), Double.parseDouble(v.getValue().toString()));
				}	
			}
		} catch (ClassNotFoundException | SQLException e) {
			AgentHelpers.logAgent(ba, Level.WARNING, LogMessages.TROUBLEINSERTINGINTERACTION.getDescription()+e);
		} 
		
	}
	
	@Override
	public boolean done() {
		return state == OK || state == KO;
	}
	
	@Override
	public int onEnd() {		
		return state;
	}
	
}