package tfg.sma.core.v2.behaviours;

import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import jade.core.behaviours.Behaviour;
import net.sourceforge.jFuzzyLogic.FIS;
import tfg.sma.core.model.Variable;
import tfg.sma.core.v2.agent.BaseAgent;
import tfg.sma.logging.LogMessages;
import tfg.sma.utility.AgentHelpers;


import java.util.Vector;
import java.util.logging.Level;

@SuppressWarnings("serial")
public class WisdomClaim extends Behaviour {

	public static final int OK = 0;
	public static final int KO = -1;
	public static final String FISFILEFOLDER = "fis/";
	
	private int state;
	private String fileName;
		
	public WisdomClaim(String fileName) {
		this.fileName = FISFILEFOLDER+fileName;	
	}
	
	@Override
	public void action() {
		state = KO;		
		BaseAgent agent = (BaseAgent)myAgent;	
		try {								
			FIS fisJ = FIS.load(fileName, false);
			JFuzzyLogicVariableInitialization(fisJ, agent);
			
		state = OK;
		} catch(java.util.NoSuchElementException e) {
			//Troublesome parsing			
			AgentHelpers.logAgent(agent, Level.WARNING, LogMessages.FCLTROUBLE.getDescription()+e);
			state = KO;					
		}
		
	}
		
	private void JFuzzyLogicVariableInitialization(FIS fis, BaseAgent agent) {
		Vector<Variable> inputVariables = new Vector<Variable>();
		Vector<Variable> outputVariables = new Vector<Variable>();
			
		Set<Entry<String,net.sourceforge.jFuzzyLogic.rule.Variable>> entrySet = fis.getFunctionBlock(null).getVariables().entrySet();		
			for(Map.Entry<String, net.sourceforge.jFuzzyLogic.rule.Variable> entry : entrySet) {
				net.sourceforge.jFuzzyLogic.rule.Variable v = entry.getValue();
				String type = v.getDefuzzifier() != null ? Variable.OUTPUT : Variable.INPUT; 		        
		        switch(type) {
				case Variable.INPUT:
					inputVariables.add(new Variable(Variable.INPUT,v.getName(),null));					
					break;
				case Variable.OUTPUT:
					outputVariables.add(new Variable(Variable.OUTPUT,v.getName(),null));
					break;
				}
			}
		   
		//Once the variables are all read, propagating vectors and fis to the agent.
		agent.getVariables().put(Variable.INPUT, inputVariables);
		agent.getVariables().put(Variable.OUTPUT, outputVariables);
		agent.getVariables().put(Variable.FUZZYISYSTEM, fis);		
	}
	
	
	
	@Override
	public boolean done() {
		return state == OK || state == KO;
	}
	
	@Override
	public int onEnd() {		
		return state;
	}

}
