package tfg.sma.core.v2.ontology;

import jade.content.onto.BasicOntology;
import jade.content.onto.Ontology;
import jade.content.onto.OntologyException;
import jade.content.schema.ConceptSchema;
import jade.content.schema.ObjectSchema;
import jade.content.schema.PredicateSchema;
import jade.content.schema.PrimitiveSchema;
import tfg.sma.core.model.Variable;

@SuppressWarnings("serial")
public class TfgSmaOntology extends Ontology {

	public static final String ONTOLOGYNAME = "tfg-sma-ontology";	
	//Concepts
	public static final String VARIABLE ="variable";
	public static final String VARIABLE_TYPE ="type";
	public static final String VARIABLE_NAME ="name";
	public static final String VARIABLE_VALUE ="value";
	
	public static final String PROPERTY ="property";
	public static final String PROPERTY_NAME ="propertyName";
	public static final String PROPERTY_VALUE ="value";
	
	//Predicates
	public static final String VARIABLE_PREDICATE ="variables";
	public static final String AGENTPROP_PREDICATE ="agentProperties";
	
	
	private static Ontology instance = new TfgSmaOntology();	
	public static Ontology getInstance() {
		return instance;
	}
	
	private TfgSmaOntology() {		
		super(ONTOLOGYNAME, BasicOntology.getInstance());		
		try {
			ConceptSchema variableConcept = new ConceptSchema(VARIABLE);
			add(variableConcept, Variable.class);			
			variableConcept.add(VARIABLE_TYPE, (PrimitiveSchema)getSchema(BasicOntology.STRING));
			variableConcept.add(VARIABLE_NAME, (PrimitiveSchema)getSchema(BasicOntology.STRING));
			variableConcept.add(VARIABLE_VALUE, (PrimitiveSchema)getSchema(BasicOntology.FLOAT));
			
			ConceptSchema propertyConcept = new ConceptSchema(PROPERTY);
			add(propertyConcept, PropertyConcept.class);
			propertyConcept.add(PROPERTY_NAME, (PrimitiveSchema)getSchema(BasicOntology.STRING));
			propertyConcept.add(PROPERTY_VALUE, (PrimitiveSchema)getSchema(BasicOntology.STRING));
						
			PredicateSchema variablesPredicate = new PredicateSchema(VARIABLE_PREDICATE);
			add(variablesPredicate, VariablesPredicate.class);						
			variablesPredicate.add(VARIABLE_PREDICATE, variableConcept,0,ObjectSchema.UNLIMITED);
												
			PredicateSchema agentInfoPredicate = new PredicateSchema(AGENTPROP_PREDICATE);
			add(agentInfoPredicate, AgentInformationPredicate.class);
			agentInfoPredicate.add(AGENTPROP_PREDICATE, propertyConcept, 0,ObjectSchema.UNLIMITED);
												
		}catch(OntologyException oe) {
			oe.printStackTrace();
		}		
	}
}


