package tfg.sma.ddbb;

import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.text.MessageFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Locale;
import java.util.TimeZone;
import java.util.Vector;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;

import tfg.sma.core.model.DataBaseDescriptor;

/**
 * Singleton to handle all DataBAse operations.
 * @author UOC-TFG
 *
 */
/**
 * @author UOC-TFG
 *
 */
public class DbManager {

	private static DbManager instance = null;
	private static Connection connection;
			
	//Default values
	private static String DRIVER = "oracle.jdbc.driver.OracleDriver";
	private static String CONNECTION = "jdbc:oracle:thin:@localhost:1521:XE";
	private static String USER = "TFG-SMA";
	private static String PASSWORD ="un5c413r0ns";
	private static Locale LOCALE = Locale.ENGLISH; //Beware, this might be trourblesome	
	private static boolean CONNECTION_OK = false;
	
		
	private DbManager() throws ClassNotFoundException, SQLException {
		connection = getDBConnection();
	}
	
	private DbManager(DataBaseDescriptor dbd) throws ClassNotFoundException, SQLException {
		if(dbd != null) {
			CONNECTION = dbd.getConnection();
			DRIVER = dbd.getDriver();
			LOCALE =  new Locale(dbd.getLocale());
			PASSWORD = dbd.getPassword();
			USER = dbd.getUser();	
		}
		connection = getDBConnection();
		
	}
	
	
	public synchronized static DbManager getInstance() throws ClassNotFoundException, SQLException {
		if(instance == null)
			instance = new DbManager();		
		return instance;
	}
	
	public synchronized static DbManager getInstance(DataBaseDescriptor dbd) throws ClassNotFoundException, SQLException {
		if(instance == null)
			instance = new DbManager(dbd);		
		return instance;
	}
	
	private static Connection getDBConnection() throws SQLException , ClassNotFoundException{
		Connection dbConnection = null;
			Locale.setDefault(LOCALE); //The DDBB have been installed in english.
			Class.forName(DRIVER);
			dbConnection = DriverManager.getConnection(CONNECTION, USER,PASSWORD);
			//We might be interested about grouping sql statements...and once all are OK, perform the commit.
			//So, commit must be done manually.
			dbConnection.setAutoCommit(false); 
			setCONNECTION_OK(true);
			return dbConnection;		
	}
	
	@SuppressWarnings("unused")
	private void  runQuery(String query) throws SQLException {		
		
		try {
			PreparedStatement p = connection.prepareStatement(query);
			p.executeQuery();
			//If no exception have been thrown, commiting.
			connection.commit();
			p.close();
		} catch (SQLException e) {						
			throw e;
			
		}		
	}
	
	private ResultSet runQueryRs(String query) throws SQLException {		
		ResultSet rs = null; 
		try {
			PreparedStatement p = connection.prepareStatement(query);
			rs=p.executeQuery();	
			//I cannot close yet the rs..because it's going to be used by the invoker.		
			return rs;
		} catch (SQLException e) {			
			throw e;
			 
		}	
	}
			
	@SuppressWarnings("unused")
	private void runQueries(String [] query) throws SQLException {		
		
		try {
			//DDL will always commit even if the autocommit is disabled.
			for(String sql : query) {
				PreparedStatement p=connection.prepareStatement(sql);
				p.executeQuery();
				p.close();
			}
			connection.commit();
			
			//If no exception have been thrown, commiting.
		} catch (SQLException e) {
			//We are interested in make the agent aware that the SQL failed.
			throw e;		
		} 
	}
	
	@SuppressWarnings("unused")
	private Long getSequenceNextValue(String sequenceName) throws SQLException   {		
		String sql = MessageFormat.format("SELECT {0}.NEXTVAL FROM DUAL", sequenceName);		
		ResultSet rset = null;
		PreparedStatement p = null;
		Long i = null;
		try {			
			synchronized (this) {
				p = connection.prepareStatement(sql);
				rset = p.executeQuery();	
			}
			if(rset.next()) {
				i = rset.getLong(1);
				rset.close();
				p.close();
			}							
			return i;
		} catch (SQLException e) {
			//We are interested in make the agent aware that the SQL failed.
			throw e;		
		} 
		
	}
	/**
	 * Method to create all the DDBB elements.
	 * @param queries
	 */
	public void prepareDDL(ArrayList<String> queries) {
		if(queries.size() != 0) {
			for(String sql : queries) {
				PreparedStatement p = null;
				try {					
					p = connection.prepareStatement(sql);
					p.executeQuery();
					p.close();
				}catch (SQLException e) {
					//If something exists, an exception will be generated
				}finally {
					try {
						p.close();
					} catch (SQLException e) {							
						//e.printStackTrace(); TODO: Shall be logged properly.
					}
				}
				continue;
			}			
		}		
	}
	
	/**
	 * Connectivity TEST
	 * @throws SQLException
	 */
	public void TestDatabase() throws SQLException {		
		String sql="SELECT SYSDATE FROM DUAL";		
		try {
			runQuery(sql);
		} catch (SQLException e) {
			setCONNECTION_OK(false);
			throw e;
		}		
	}
	
	private String quotedString(String value) {
		return "'"+value+"'";
	}
	
	private String toOracleTimeStamp(Timestamp instant) {
		SimpleDateFormat sdf = new SimpleDateFormat("dd-MMM-YY HH.mm.ss", Locale.ENGLISH);
		sdf.setTimeZone(TimeZone.getTimeZone("UTC"));
		String formattedInstant = sdf.format(instant).toUpperCase().toString();
		return MessageFormat.format("TO_TIMESTAMP(''{0}'',''DD-MON-YY HH24.MI.SS'')", formattedInstant);
	}
		
	private String insertTfgSmaSessionSQL(Long sesionId, Timestamp startDate, boolean simulation, String status ) {	
		return MessageFormat.format("INSERT INTO TFG_SMA_SESSION (ISESSIONID, TSTARTDATE, ISIMULATION, SSIMULATION_STATUS) VALUES({0,number,#},{1},{2},{3})", sesionId, toOracleTimeStamp(startDate), simulation==false?0:1, quotedString(status));		
	}
	
	private String closeTfgSmaSessionSQL(Long sesionId, Timestamp endDate) {	
		return MessageFormat.format("UPDATE TFG_SMA_SESSION SET SSIMULATION_STATUS ={0},TSENDDATE ={2} WHERE ISESSIONID ={1,number,#}", quotedString(DbConstants.ENDED) , sesionId,toOracleTimeStamp(endDate) );		
	}
	
	private String insertTfgSmaIntreactionSQL(Long interactionId, Long sesionId, String source, String target, Timestamp interactionDate, int inferenceId, String fclFileName) {	
		return MessageFormat.format("INSERT INTO TFG_SMA_SESSION_INTERACTION (IINTERACTIONID, ISESSIONID, SSOURCE, STARGET, TINTERACTIONDATE, IINFERENCEID, SFCLFILENAME ) VALUES({0,number,#},{1,number,#},{2},{3},{4},{5,number,#},{6})", interactionId, sesionId, quotedString(source), quotedString(target),toOracleTimeStamp(interactionDate),inferenceId ,quotedString(fclFileName));		
	}
	
	
	private String insertTfgSmaInteractVariableSQL(Long variableId, Long interactionId, String name, String type, double value) {	
		return MessageFormat.format("INSERT INTO TFG_SMA_INTERACTION_VARIABLE (IVARIABLEID, IINTERACTIONID, SNAME, STYPE, FVALUE) VALUES({0,number,#},{1,number,#},{2},{3},{4})", variableId, interactionId, quotedString(name), quotedString(type), quotedString(Double.toString(value)));		
	}
	
	private String selectTfgSmaInteractionByIdSQL(Long sessionId) {	
		return MessageFormat.format("SELECT * FROM TFG_SMA_SESSION_INTERACTION WHERE ISESSIONID = {0,number,#}", sessionId);				
	}
	
	
	@SuppressWarnings("serial")
	public void createDDBBTables() {		
		prepareDDL(new ArrayList<String>() {{
			add(DbConstants.TFGSMA_SESSION_CREATE);
			add(DbConstants.TFGSMA_INTERACTION_CREATE);
			add(DbConstants.TFGSMA_INTER_VARIABLE_CREATE);
			add(DbConstants.TFGSMA_SESSIONID_SEQ_CREATE);
			add(DbConstants.TFGSMA_INTERACTIONID_SEQ_CREATE);
			add(DbConstants.TFGSMA_VARIABLEID_SEQ_CREATE);
		}});
		
	}
	
	
	
	public synchronized Long newTfgSmaSession(boolean simulation, String status) throws SQLException {
		Long sessionId = this.getSequenceNextValue(DbConstants.TFGSMA_SESSIONID_SEQ);		
		if(sessionId!=null) {			
			Timestamp now = new Timestamp(System.currentTimeMillis());
			String sql = insertTfgSmaSessionSQL(sessionId,now,simulation,status);
			runQuery(sql);
			return sessionId;
		}
		return null;
	}
	
	public synchronized void closeTfgSmaSession(Long sessionId) throws SQLException {
		if(sessionId!=null) {			
			Timestamp now = new Timestamp(System.currentTimeMillis());
			String sql = closeTfgSmaSessionSQL(sessionId, now);
			runQuery(sql);			
		}		
	}
	
	
	
	public synchronized Long newTfgSmaInteraction(Long sessionId, String source, String target, int inferenceId, String fclFileName) throws SQLException {
		Long interactionId = this.getSequenceNextValue(DbConstants.TFGSMA_INTERACTIONID_SEQ);		
		if(interactionId!=null) {			
			Timestamp now = new Timestamp(System.currentTimeMillis());
			String sql = insertTfgSmaIntreactionSQL(interactionId, sessionId, source ,target,now, inferenceId, fclFileName);
			runQuery(sql);
			return interactionId;
		}
		return null;
	}
	
	public synchronized void newTfgSmaInterVariable(Long interactionId, String name, String type, double value) throws SQLException {
		Long variableId = this.getSequenceNextValue(DbConstants.TFGSMA_VARIABLEID_SEQ);		
		if(variableId!=null) {	
			String sql = insertTfgSmaInteractVariableSQL(variableId, interactionId, name ,type,value);
			runQuery(sql);	
		}		
	}
	
	public synchronized ResultSet getTfgSmaInteractions(Long sessionId) throws SQLException {
		if(sessionId!=null) {
			String sql = selectTfgSmaInteractionByIdSQL(sessionId);			
			return runQueryRs(sql);
		}
		return null;			
	}

	/**
	 * Credit to Paul Vargas - StackOverflow.
	 * @param rs
	 * @return
	 * @throws SQLException
	 */
	public static DefaultTableModel buildTableModel(ResultSet rs)
	        throws SQLException {

	    ResultSetMetaData metaData = rs.getMetaData();

	    // names of columns
	    Vector<String> columnNames = new Vector<String>();
	    int columnCount = metaData.getColumnCount();
	    for (int column = 1; column <= columnCount; column++) {
	        columnNames.add(metaData.getColumnName(column));
	    }

	    // data of the table
	    Vector<Vector<Object>> data = new Vector<Vector<Object>>();
	    while (rs.next()) {
	        Vector<Object> vector = new Vector<Object>();
	        for (int columnIndex = 1; columnIndex <= columnCount; columnIndex++) {
	            vector.add(rs.getObject(columnIndex));
	        }
	        data.add(vector);
	    }
	    
	  //I'm dont with the rs, closing it.
	    rs.getStatement().close();
	    rs.close();
	    return new DefaultTableModel(data, columnNames);

	}
	
	public synchronized static void  refreshTableModel(JTable table, ResultSet rs)
	        throws SQLException {

		try {
		DefaultTableModel tm = (DefaultTableModel) table.getModel();
		ResultSetMetaData metaData = rs.getMetaData();

		    // names of columns
		    Vector<String> columnNames = new Vector<String>();
		    int columnCount = metaData.getColumnCount();
		    for (int column = 1; column <= columnCount; column++) {
		        columnNames.add(metaData.getColumnName(column));
		    }

		    // data of the table
		    Vector<Vector<Object>> data = new Vector<Vector<Object>>();
		    while (rs.next()) {
		        Vector<Object> vector = new Vector<Object>();
		        for (int columnIndex = 1; columnIndex <= columnCount; columnIndex++) {
		            vector.add(rs.getObject(columnIndex));
		        }
		        data.add(vector);
		    }		   
		    tm.setDataVector(data, columnNames);
		}catch(Exception e) {
			
		}finally {
			 rs.getStatement().close();
			 rs.close();
		}

	}
		
	public static boolean isCONNECTION_OK() {
		return CONNECTION_OK;
	}

	public static void setCONNECTION_OK(boolean cONNECTION_OK) {
		CONNECTION_OK = cONNECTION_OK;
	}
	
	
	
	
	
}
