package tfg.sma.gui;

import java.awt.*;
import java.awt.event.*;
import java.util.Calendar;
import java.util.Map;
import java.util.Vector;
import java.util.logging.Level;
import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import tfg.sma.core.model.Variable;
import tfg.sma.core.v2.agent.BaseAgent;
import tfg.sma.core.v2.behaviours.Inference;
import tfg.sma.logging.LogMessages;
import tfg.sma.utility.AgentHelpers;

@SuppressWarnings("serial")
public class BaseAgentGui extends JFrame {
	
	private BaseAgent myAgent;
	//GUI Elements
	private JPanel infoPanel = new JPanel();
	private JPanel inputPanel = new JPanel();
	private JPanel outputPanel = new JPanel();
	private JPanel agentVariablesPanel = new JPanel();
	private JTextArea logTextArea = new JTextArea(15,1);
	private JScrollPane logPanel = new JScrollPane(logTextArea);
	private JButton addButton = new JButton(GuiConstants.VALUESTOAGENT);
	private JButton askButton = new JButton(GuiConstants.INFERENCEBUTTON);		
	
	
		
	public BaseAgentGui(BaseAgent a){
		
		super(a.getLocalName());
		myAgent = a;		
		//Preparing GUI elements.
		setVanillaGuiProperties();		
		prepareBaseAgentGui();
		prepareButtons();
		
		//Once all the gui elements has been prepared, the panels are set within this Frame.
		this.add(infoPanel, BorderLayout.NORTH);		
		this.add(agentVariablesPanel, BorderLayout.CENTER);
		this.add(logPanel, BorderLayout.SOUTH);
		
		this.pack();
	}
		
	public ActionListener propagateGuiValuesToAgent() {		
		return new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				for(Map.Entry<String, Object> entry : myAgent.getVariables().entrySet()) {
					String values = "";
					//Propagating only input values to the agent hashmap.
					switch(entry.getKey()){					
						case Variable.INPUT:
							@SuppressWarnings("unchecked") 
							Vector<Variable> inputVector = (Vector<Variable>)entry.getValue();				
							for(Variable v: inputVector) {
								try{
									v.setValue(Double.parseDouble(((JTextField)inputPanel.getClientProperty(v.getName())).getText()));
									values += v.getName() +"="+ v.getValue()+" ";								
									notifyInputLabel(v);
								}catch(NumberFormatException ex) {
									AgentHelpers.logAgent(myAgent, Level.WARNING, myAgent.getAID().getLocalName()+LogMessages.PARSINGEXCEPTION.getDescription()+v.getName()+" "+ex);
								}
								continue;
							}
							AgentHelpers.logAgent(myAgent, Level.INFO, myAgent.getAID().getLocalName()+LogMessages.AGENTMANUALLYFED.getDescription()+": "+values);
						break;
					}
				}
				askButton.setEnabled(true);
			}			
		};
		
	}

	public ActionListener performOwnInference(BaseAgent agent) {
		return new ActionListener() {			
			@Override
			public void actionPerformed(ActionEvent e) {				
				agent.addBehaviour(new Inference(agent.getDataStore(), true));
			}			
		};
	}


	public void notifyGuiNewValues() {
		for(Variable v : myAgent.getAgentInputVariables()) {
			((JTextField)inputPanel.getClientProperty(v.getName())).setText(v.getValue().toString());
			notifyInputLabel(v);
		}
		
		for(Variable v : myAgent.getAgentOutputVariables()) {
			((JTextField)outputPanel.getClientProperty(v.getName())).setText(v.getValue().toString());
		}
		
	}



	public void showGui() {
		pack();
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		int centerX = (int)screenSize.getWidth() / 2;
		int centerY = (int)screenSize.getHeight() / 2;
		setLocation(centerX - getWidth() / 2, centerY - getHeight() / 2);
		super.setVisible(true);
	}



	public void logActivity(String m) {
		logTextArea.append(Calendar.getInstance().getTime()+" "+m+"\n");
	}



	private void setVanillaGuiProperties() {
		
		this.setIconImage(new ImageIcon(GuiConstants.UOCICON).getImage());
		logTextArea.setEditable(false);		
		infoPanel.setBorder(BorderFactory.createTitledBorder(GuiConstants.AGENTINFORMATION));
		inputPanel.setBorder(BorderFactory.createTitledBorder(GuiConstants.INPUTVALUES));
		outputPanel.setBorder(BorderFactory.createTitledBorder(GuiConstants.OUTPUTVALUES));
		logPanel.setBorder(BorderFactory.createTitledBorder(GuiConstants.ACTIVITY));
		logPanel.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		
	}
	
	
	@SuppressWarnings("unchecked")
	private void prepareBaseAgentGui() {
		
		infoPanel.add(new Label(myAgent.getOutputInferenceDescriptor().getFileName()));		
		if(myAgent.getAgentDescriptor().getAskPeriod() > 0) {
			infoPanel.add(new JLabel(GuiConstants.ASKING, new ImageIcon(GuiConstants.ASKINGICON), JLabel.CENTER));			
		}
			
		
		//Variable processing to enrich the base agent GUI accordingly to its FCL.
		for(Map.Entry<String, Object> entry : myAgent.getVariables().entrySet()) {			
			switch(entry.getKey()) {			
			//Adding to the gui only the input variables
			case Variable.INPUT:
				Vector<Variable> inputVector = (Vector<Variable>)entry.getValue();
				//inputPanel.setLayout(new GridLayout(inputVector.size()+1, 3));		
				inputPanel.setLayout(new BoxLayout(inputPanel, BoxLayout.PAGE_AXIS));
				//Gathering all the textfields...To enable/Disable the propagate button				
				JTextField[] f = new JTextField[inputVector.size()];
				int i= 0;
				
				for(Variable v: inputVector) {					
					inputPanel.add(new JLabel(v.getName()));
					
					//Creating the input field and its validation listener.
					JTextField fieldValue = new JTextField();
					fieldValidationListener(fieldValue);
					
					inputPanel.putClientProperty(v.getName(), fieldValue);
					inputPanel.add(fieldValue);
					f[i] = fieldValue;
					i++;
					
					//Labels to display currently propagated values to the agent's input.
					JLabel currentValueLabel = new JLabel(v.getValue().toString());					
					registerJLabel(currentValueLabel, v.getName());
					
				}				
				commonListener(f, myAgent);
				break;
			case Variable.OUTPUT:
				Vector<Variable> outputVector = (Vector<Variable>)entry.getValue();
				//outputPanel.setLayout(new GridLayout(outputVector.size()+1, 2));
				outputPanel.setLayout(new BoxLayout(outputPanel, BoxLayout.PAGE_AXIS));
				for(Variable v: outputVector) {
					outputPanel.add(new JLabel(v.getName()));
					JTextField fieldValue = new JTextField();
					fieldValue.setEditable(false);
					outputPanel.putClientProperty(v.getName(), fieldValue);
					outputPanel.add(fieldValue, BorderLayout.CENTER);
				}
			}
		}
		agentVariablesPanel.setLayout(new BoxLayout(agentVariablesPanel, BoxLayout.LINE_AXIS));
		agentVariablesPanel.add(inputPanel);
		agentVariablesPanel.add(outputPanel);
	}
	
	
	private void prepareButtons() {
		addButton.addActionListener(propagateGuiValuesToAgent());
		addButton.setEnabled(false);
		askButton.addActionListener(performOwnInference(myAgent));
		askButton.setEnabled(false);
		inputPanel.add(addButton, BorderLayout.CENTER);
		outputPanel.add(askButton, BorderLayout.SOUTH);
	}
	
	
	
	private void notifyInputLabel(Variable v) {
		((JLabel)inputPanel.getClientProperty(v.getName()+"_label")).setText(v.getValue().toString());
	}
	
	private void registerJLabel(JLabel label, String valueName) {
		label.setHorizontalAlignment(SwingConstants.CENTER);
		inputPanel.putClientProperty(valueName+"_label", label);
	}

	//Trying to parse.
	private void fieldValidationListener(JTextField fieldValue) {
		fieldValue.addKeyListener(new KeyListener() {												
			@Override
			public void keyReleased(KeyEvent e) {
				try {
					Double.parseDouble(fieldValue.getText());
					fieldValue.setForeground(Color.BLACK);
					
				}catch (NumberFormatException ne) {
					
					fieldValue.setForeground(Color.RED);
				}							
			}						
			@Override
			public void keyPressed(KeyEvent e) {												
			}

			@Override
			public void keyTyped(KeyEvent e) {
			}
		});
	}
	
	//Adding the same listener to all the JTextFields, thus the invoked method will be able to check all the values of the JTextFields.
	private void commonListener(JTextField [] f, BaseAgent agent) {		
		DocumentListener d = new DocumentListener() {			
			@Override
			public void removeUpdate(DocumentEvent e) {
				enableOrNotButtons(f, agent);
			}			
			@Override
			public void insertUpdate(DocumentEvent e) {
				enableOrNotButtons(f, agent);				
			}			
			@Override
			public void changedUpdate(DocumentEvent e) {
				enableOrNotButtons(f, agent);
			}
		};
		for(JTextField jf : f ) {
			jf.getDocument().addDocumentListener(d);
		}		
	}

	private void enableOrNotButtons(JTextField[] f, BaseAgent agent) {
		boolean enable = true;		
		for(JTextField jf : f ) {
			if(jf.getText().equals("")){
				enable = false;
				break;
			}			
		}	
		addButton.setEnabled(enable);
		
		for(Variable v: agent.getAgentInputVariables()) {
			if(v.getValue().equals(Variable.NULL)) {
				enable = false;
				break;
			}
			enable = true;
		}
		askButton.setEnabled(enable);
	}
	
}
