package tfg.sma.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.SQLException;
import java.util.Timer;
import java.util.TimerTask;
import java.util.logging.Level;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.RowFilter;
import javax.swing.RowSorter;
import javax.swing.SwingWorker;
import javax.swing.UIManager;
import javax.swing.WindowConstants;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;
import jade.core.AID;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.lang.acl.ACLMessage;
import tfg.sma.core.v2.agent.HiveAgent;
import tfg.sma.ddbb.DbManager;
import tfg.sma.logging.LogMessages;
import tfg.sma.utility.AgentHelpers;
import tfg.sma.utility.PlatformHelpers;



@SuppressWarnings("serial")
public class HiveAgentGui extends JFrame{

	private final Long REFRESHPERIOD = 5000L;
	
	//Gui Elements
	private JPanel leftPanel = new JPanel(new BorderLayout());
	private JPanel rightPanel = new JPanel(new BorderLayout());
	private JPanel searchPanel = new JPanel(new BorderLayout());
	private JPanel dataPanel = new JPanel(new BorderLayout());
	private JPanel buttonsPanel = new JPanel();
	private JTextField filterField = new JTextField();
	private JButton greetBaseAgent;
	private JButton greetRmaAgent;
	private JTable agentsTable;
	private JTable dataTable;
	
	//Functional elements
	private AID selectedAgent;
	private Long sessionId;
	

	public HiveAgentGui(HiveAgent a, DFAgentDescription [] agents) {		
		
		super(a.getLocalName());
		sessionId = a.getSessionId();		
		//Basic Frame properties
		setVanillaGuiProperties();		
		//LEFT side of the GUI
		searchPanel.add(new JLabel(GuiConstants.SEARCHCRITERIA), BorderLayout.WEST);
		searchPanel.add(filterField);
		leftPanel.add(prepareAgentsTable(agents,a), BorderLayout.NORTH);
		leftPanel.add(searchPanel, BorderLayout.SOUTH);
		
		//RIGHT side of the GUI
		if(DbManager.isCONNECTION_OK())
			dataPanel.add(prepareAgentsDataTable(a), BorderLayout.NORTH);
		else dataPanel.add(setNoInteractionsDDBB(), BorderLayout.CENTER);
			
			
		prepareButtons(a);		
		rightPanel.add(dataPanel,BorderLayout.LINE_START);
		rightPanel.add(buttonsPanel, BorderLayout.SOUTH);
		
		//Adding to the Fram both left and right gui sides.
		add(leftPanel, BorderLayout.LINE_START);
		add(rightPanel);
		
		this.pack();
		
		//Periodically, a SwingWorker it's created to refresh the dataTable from the DDBB.
		if(DbManager.isCONNECTION_OK())
			refreshAgenstDataTable();		
	}
	
	private void setVanillaGuiProperties() {
		this.setResizable(false);
		this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		this.setIconImage(new ImageIcon(GuiConstants.UOCICON).getImage());
		
		leftPanel.setBorder(BorderFactory.createTitledBorder(GuiConstants.BORDER_AGENTS_TITTLE));
		rightPanel.setBorder(BorderFactory.createTitledBorder(GuiConstants.BORDER_DATA_TITTLE));
	}
	
	private JScrollPane prepareAgentsTable(DFAgentDescription [] agents, HiveAgent a) {
		final String[] columnNames = {GuiConstants.NAMEAGENTCOL};		
		Object[][] data = null;
		if(agents != null) {
			data = new Object[agents.length][2];
			for(int i = 0; i<agents.length;i++) {
				data[i][0] = agents[i].getName().getLocalName();				
				data[i][1] = agents[i].getName();
			}			
		}		
		agentsTable = new JTable(data, columnNames);
		agentsTable.setDefaultEditor(Object.class, null);
		
		agentsTable.setFillsViewportHeight(true);
		agentsTable.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
		
		this.listenerFiltreTauleAgents(agentsTable, filterField);
		this.listenerRowAgentsTable(agentsTable, a);
		
		
		JScrollPane tableScrollPane = new JScrollPane(agentsTable);
		tableScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		return tableScrollPane;
	}
	
	
	private JScrollPane prepareAgentsDataTable(HiveAgent a) {		
			CreateGuiDataTable(a);
			JScrollPane dataTableScrollPane = new JScrollPane(dataTable);
			dataTableScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
			dataTable.setFillsViewportHeight(true);
			dataTable.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);			
			return dataTableScrollPane;	
		
	}
	
	
	private void prepareButtons(HiveAgent a) {
		//Exit button
		JFrame frame = this;
		this.addWindowListener(new java.awt.event.WindowAdapter() {
		    @Override
		    public void windowClosing(java.awt.event.WindowEvent windowEvent) {
		        if (JOptionPane.showConfirmDialog(frame, 
		            GuiConstants.EXIT, GuiConstants.EXITT_TITTLE, 
		            JOptionPane.YES_NO_OPTION,
		            JOptionPane.QUESTION_MESSAGE) == JOptionPane.YES_OPTION){
		            //Trying to close the Work session and shutting down JADE.
		        	try {
		        		if(DbManager.isCONNECTION_OK())
							DbManager.getInstance().closeTfgSmaSession(a.getSessionId());
					} catch (ClassNotFoundException | SQLException e) {					
						e.printStackTrace();
					} finally {
						PlatformHelpers.shutDownJadePlatform(a);	
					}		        	
		        }
		    }
		});
		
		buttonsPanel.setLayout(new BoxLayout(buttonsPanel ,BoxLayout.LINE_AXIS));
		
		greetBaseAgent = new JButton(GuiConstants.ACCESAGENT);
		listenerGreetBaseAgent(greetBaseAgent, a);
		greetBaseAgent.setEnabled(false);
				
		greetRmaAgent = new JButton(GuiConstants.ACCESRMA);
		listenerGreetRmaAgent(greetRmaAgent, a);
		greetRmaAgent.setAlignmentX(Component.CENTER_ALIGNMENT);
		buttonsPanel.add(greetBaseAgent);
		buttonsPanel.add(greetRmaAgent);		
	}
	
	private JLabel setNoInteractionsDDBB() {		
		JLabel noInteractionsDDBB = new JLabel(GuiConstants.NOINTERACTIONSDDBB, UIManager.getIcon("OptionPane.warningIcon"), JLabel.CENTER);		
		return noInteractionsDDBB;
	}
	
	
	private void sendToBaseAgent(HiveAgent agent) {
		ACLMessage m = new ACLMessage(ACLMessage.QUERY_IF);
		m.addReceiver(selectedAgent);
		agent.send(m);
	}
			
	
	private void CreateGuiDataTable(HiveAgent ha) {
		try {
			dataTable = new JTable(DbManager.buildTableModel(DbManager.getInstance().getTfgSmaInteractions(ha.getSessionId())));
			dataTable.setDefaultEditor(Object.class, null);
		} catch (ClassNotFoundException e) {
			AgentHelpers.logAgent(ha, Level.SEVERE, LogMessages.HIVEAGENTDATATABLEEXCEPTION.getDescription()+e);
		} catch (SQLException e) {
			AgentHelpers.logAgent(ha, Level.SEVERE, LogMessages.HIVEAGENTDATATABLEEXCEPTION.getDescription()+e);
		}		
	}
		
	
	private void refreshAgenstDataTable() {
		Timer t = new Timer();
	    t.schedule(new TimerTask() {
	        @Override public void run() {
	        	RefreshAgentDataTableWorker rgt = new RefreshAgentDataTableWorker();
	        	rgt.execute();
	        }	        	      
	    }, 0L, REFRESHPERIOD);
				
	}
		
	/**
	 * Listener que reacciona cada cop que un registre de la taula d'agents es seleccionat.
	 * @param table
	 */
	private void listenerRowAgentsTable(JTable table, HiveAgent agent) {
		//Selecci d'un unic registre
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		//S'afegeix el listener.
		table.getSelectionModel().addListSelectionListener(new ListSelectionListener() {			
			@Override
			public void valueChanged(ListSelectionEvent e) {				
				//S'ignoren la resta d'events
                if (e.getValueIsAdjusting()) return;
				                               
                ListSelectionModel lsm = (ListSelectionModel)e.getSource();
                
                if (lsm.isSelectionEmpty()) {
                  
                } else {
                    int selectedRow = lsm.getMinSelectionIndex();                   
                    selectedAgent = (AID) table.getModel().getValueAt(table.convertRowIndexToModel(selectedRow),1);
                    greetBaseAgent.setEnabled(true);                                        
                }				
			}
		});				
	}
	
		
	/**
	 * Listener que reacciona cada cop que el filterField es editat, filtrant els agents coincidents de la taula.
	 * @param table
	 * @param filterField
	 */
	private void listenerFiltreTauleAgents(JTable table, JTextField filterField ) {
		
		TableRowSorter<TableModel> trowSorter= new TableRowSorter<>(table.getModel());
		table.setRowSorter(trowSorter);
		filterField.getDocument().addDocumentListener(new DocumentListener() {					
			
			private void filterAndSorter() {
				String inputText = filterField.getText();
                if (inputText.trim().length() == 0) {
                	trowSorter.setRowFilter(null);
                } else {
                	trowSorter.setRowFilter(RowFilter.regexFilter("(?i)" + inputText));
                }
                                            
			}			
			@Override
			public void removeUpdate(DocumentEvent e) {
				filterAndSorter();				
			}
			
			@Override
			public void insertUpdate(DocumentEvent e) {
				filterAndSorter();			}
			
			@Override
			public void changedUpdate(DocumentEvent e) {
				filterAndSorter();			}
		});		
		
	}
	
	private void listenerGreetBaseAgent(JButton actionButton, HiveAgent ha) {
		actionButton.addActionListener(new ActionListener() {			
			@Override
			public void actionPerformed(ActionEvent e) {
				sendToBaseAgent(ha);				
			}
		});
	}
	
	private void listenerGreetRmaAgent(JButton actionButton, HiveAgent ha) {
		actionButton.addActionListener(new ActionListener() {			
			@Override
			public void actionPerformed(ActionEvent e) {
				PlatformHelpers.createRmaAgent(ha);				
			}
		});		
	}
		
	/**
	 * Swingworker to ensure thread safe operations.
	 * @author UOC-TFG
	 *
	 */
	private class RefreshAgentDataTableWorker extends SwingWorker<Void, String>{
		@Override
		protected Void doInBackground() throws Exception {
			try {							
				DbManager.refreshTableModel(dataTable, DbManager.getInstance().getTfgSmaInteractions(sessionId));				
			} catch (ClassNotFoundException | SQLException e) {				
				e.printStackTrace();
			}	
			return null;
		}}
	
	
	public void showGui() {
		pack();
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		int centerX = (int)screenSize.getWidth() / 2;
		int centerY = (int)screenSize.getHeight() / 2;
		setLocation(centerX - getWidth() / 2, centerY - getHeight() / 2);
		super.setVisible(true);
	}
}
