package tfg.sma.utility;

import java.util.logging.Level;
import jade.core.Agent;
import jade.domain.FIPAAgentManagement.Property;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import tfg.sma.core.model.Variable;
import tfg.sma.core.v2.agent.BaseAgent;
import tfg.sma.core.v2.agent.HiveAgent;

public class AgentHelpers {

	public static final String OWNERSHIP = "TFG-SMA";	
	/**
	 * Service description provided.
	 * 
	 * Parameters: agent, registering and easyThere
	 * 
	 * easyThere: IF true, inputInferenceId property to search other agents won't be used. 
	 * 
	 * @param agent
	 * @param registering
	 * @return
	 */
	public static ServiceDescription CannonicalServiceDescription(Agent agent, boolean registering, boolean easyThere) {
		ServiceDescription sd = new ServiceDescription();				
		switch(agent.getClass().getSimpleName()) {
		case HiveAgent.AGENTTYPE: //almighty father of agents.
			sd.setName(HiveAgent.AGENTGENERICNAME);
			sd.setType(HiveAgent.AGENTTYPE);
			sd.setOwnership(OWNERSHIP);			
			break;
		case BaseAgent.AGENTTYPE: //Generic Agents		
			BaseAgent ba = (BaseAgent) agent;
			sd.setName(BaseAgent.AGENTGENERICNAME);
			sd.setType(BaseAgent.AGENTTYPE);
			sd.setOwnership(OWNERSHIP);						
			if(registering == true) {
				//Registering to the DF with the output variables of the agent...to let other agents found us. 
				// Every output variable of the associated fis will be available to other agents 
				for (Variable v : ba.getAgentOutputVariables()) {
					Property p = new Property(v.getName(), v.getName());			
					sd.addProperties(p);
				}
				//This property populates exactly the inference id offered by this agent...
				sd.addProperties(new Property("id",ba.getAgentDescriptor().getId()));
				sd.addProperties(new Property("InferenceOffered",ba.getOutputInferenceDescriptor().getId()));				
			}else if (registering == false) {
				//Searching	 - We are interesting on the agents that can offer the InferenceOffered configured on the descriptor and the ones outputing 
				//our input variables.
				for(Variable v : ba.getAgentInputVariables()){					
					Property p = new Property(v.getName(), v.getName());			
					sd.addProperties(p);
				}
				if(!easyThere) {
					//flag to look for agents only by variables names and not by the supposed inferenceOffered.
					sd.addProperties(new Property("InferenceOffered",ba.getVariables().get(Variable.INPUTFIS)));					
				}									
			}			
			break;
		}
		return sd;
	}
	
	public static ServiceDescription VanilaBaseAgentServiceDescription() {
		ServiceDescription sd = new ServiceDescription();
		sd.setName(BaseAgent.AGENTGENERICNAME);
		sd.setType(BaseAgent.AGENTTYPE);
		sd.setOwnership(OWNERSHIP);
		return sd;
	}
	
	/**
	 * General logging.
	 * @param agent
	 * @param l
	 * @param message
	 */
	public static void logAgent(Agent agent,Level l, String message) {		
		switch(agent.getClass().getSimpleName()) {
		case HiveAgent.AGENTTYPE:
			((HiveAgent)agent).getLogger().log(l,message);
			break;			
		case BaseAgent.AGENTTYPE:
			BaseAgent ba = (BaseAgent) agent;
			if(ba.getAgentJadeGui()!= null) {
				//Quite dirty, but works.TODO: Review, an exception with all this crap could mask other exceptions.
				/*String [] aMessage = Arrays.copyOfRange(message.split("\\|"), 3, message.split("\\|").length); //Removing own identification data
				String reducedMessage = Arrays.toString(aMessage);
				String finalMessage = reducedMessage.replace("[","").replace("]", "");
				ba.getAgentJadeGui().logActivity(finalMessage);  */
				ba.getAgentJadeGui().logActivity(message);
			}				
			ba.getLogger().log(l, message);
			break;
		}		
	}
	
	
}
