package tfg.sma.utility;

import java.io.File;
import java.io.IOException;
import javax.xml.XMLConstants;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import org.xml.sax.SAXException;
import tfg.sma.core.model.SmaDescriptor;

/**
 * Read XML Descriptor file.
 */
public class SmaDescriptorProvider {
	
	static final String descriptor = "config\\Descriptor.xml";		
	static final String descriptorxsd = "config\\DescriptorSchema.xsd";
	
	
	/**
	 * Method to have a Java Object representation of the SMA xml configuration file. It's validated against the XSD as well.
	 * @return Description
	 * @throws ParserConfigurationException
	 * @throws SAXException
	 * @throws IOException
	 * @throws JAXBException
	 */
	public static SmaDescriptor getDescriptor() throws ParserConfigurationException, SAXException, IOException, JAXBException {
		JAXBContext jc = JAXBContext.newInstance(SmaDescriptor.class);
		Unmarshaller umsh = jc.createUnmarshaller();
		umsh.setSchema(descriptorValidator());		
		return (SmaDescriptor) umsh.unmarshal(new File(SmaDescriptorProvider.descriptor));		
	}	

	/**
	 * Method to retrieve the validation schema xsd
	 * @return SMA Descriptor Schema
	 * @throws SAXException
	 * @throws IOException
	 */
	private static Schema descriptorValidator() throws SAXException, IOException {		
		String language = XMLConstants.W3C_XML_SCHEMA_NS_URI;
		SchemaFactory factory = SchemaFactory.newInstance(language);		
		return factory.newSchema(new File(descriptorxsd));		
	}

	
	
	
}
